/****************************************************************************
* MeshLab                                                           o o     *
* A versatile mesh processing toolbox                             o     o   *
*                                                                _   O  _   *
* Copyright(C) 2005                                                \/)\/    *
* Visual Computing Lab                                            /\/|      *
* ISTI - Italian National Research Council                           |      *
*                                                                    \      *
* All rights reserved.                                                      *
*                                                                           *
* This program is free software; you can redistribute it and/or modify      *
* it under the terms of the GNU General Public License as published by      *
* the Free Software Foundation; either version 2 of the License, or         *
* (at your option) any later version.                                       *
*                                                                           *
* This program is distributed in the hope that it will be useful,           *
* but WITHOUT ANY WARRANTY; without even the implied warranty of            *
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the             *
* GNU General Public License (http://www.gnu.org/licenses/gpl.txt)          *
* for more details.                                                         *
*                                                                           *
****************************************************************************/

#include <QtGui>

#include <math.h>
#include <limits>
#include <stdlib.h>
#include "decorate_section.h"
#include <QGLWidget>
#include <wrap/gl/addons.h>

#include <meshlab/glarea.h>
#include <common/pluginmanager.h>
#include <vcg/complex/algorithms/intersection.h>


using namespace vcg;
using namespace std;

typedef pair<Point3f,Color4b> PointPC; // this type is used to have a simple coord+color pair to rapidly draw non manifold faces

QString SampleMeshDecoratePlugin::decorationInfo(FilterIDType id) const
{
  switch(id)
  {
  case DP_SHOW_SECTION :      return tr("Draw a section");
  }
  assert(0);
  return QString();
}
QString SampleMeshDecoratePlugin::decorationName(FilterIDType id) const
{
	switch(id)
	{
	case DP_SHOW_SECTION :      return tr("Draw a section");
	}
	assert(0);
	return QString();
}

void SampleMeshDecoratePlugin::initGlobalParameterSet(QAction *action, RichParameterSet &parset)
{
  switch(ID(action)){
  case DP_SHOW_SECTION :
      parset.addParam(new RichFloat(this->SectionHeightParam(),0.0,"Section Height","The intercept of the section plane with the chosen axis"));
      parset.addParam(new RichEnum(this->SectionAxisParam(),0,QStringList()<<"X"<<"Y"<<"Z","Axis","Axis"));
    break;
  }
}		
		
bool SampleMeshDecoratePlugin::startDecorate( QAction * action, MeshDocument &md, RichParameterSet * parset, GLArea * gla)
{
  switch(ID(action)){
    case DP_SHOW_SECTION :
    {
      CMeshO em;
      CMeshO &m = md.mm()->cm;
      Plane3f pl;
      float off = parset->getFloat(this->SectionHeightParam());
      pl.Set(Point3f(0,1,0),off);
      vcg::IntersectionPlaneMesh(m,pl,em);


      CMeshO::PerMeshAttributeHandle< vector<PointPC> > secVecH = vcg::tri::Allocator<CMeshO>::GetPerMeshAttribute< vector<PointPC> >(m,"SectionVertexVector");

      if(!vcg::tri::Allocator<CMeshO>::IsValidHandle(m,secVecH))
        secVecH=vcg::tri::Allocator<CMeshO>::AddPerMeshAttribute< vector<PointPC> >(m,std::string("SectionVertexVector"));
      vector<PointPC> *secVec = &secVecH();
      secVec->clear();
      for(CMeshO::VertexIterator vi=em.vert.begin();vi!=em.vert.end();++vi)
      {
        secVec->push_back(make_pair(vi->cP(),Color4b::Blue));
      }

    }
    break;
  }
	return true;
}

void SampleMeshDecoratePlugin::decorate(QAction *a, MeshDocument &md, RichParameterSet * parset,GLArea *gla, QPainter * )
{
  static QString lastname("unitialized");
	switch(ID(a))
	{
	  case DP_SHOW_SECTION :
		{
	  CMeshO &m = md.mm()->cm;
	  CMeshO::PerMeshAttributeHandle< vector<PointPC> > bvH = vcg::tri::Allocator<CMeshO>::GetPerMeshAttribute<vector<PointPC> >(m,"SectionVertexVector");
	  if(vcg::tri::Allocator<CMeshO>::IsValidHandle (m, bvH))
	  {
		vector<PointPC> *BVp = &bvH();
		if (BVp->size() != 0)
		{
		  glPushAttrib(GL_ENABLE_BIT|GL_VIEWPORT_BIT|	  GL_CURRENT_BIT |  GL_DEPTH_BUFFER_BIT);
		  glDisable(GL_LIGHTING);
		  glDepthFunc(GL_LEQUAL);
		  glEnable(GL_LINE_SMOOTH);
		  glEnable(GL_BLEND);
		  glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
		  glLineWidth(1.f);
		  glColor(Color4b::Green);
		  glDepthRange (0.0, 0.999);
		  glEnableClientState (GL_VERTEX_ARRAY);
		  glVertexPointer(3,GL_FLOAT,sizeof(PointPC),&(BVp->begin()[0]));
		  glDrawArrays(GL_LINES,0,BVp->size());
		  glDisableClientState (GL_VERTEX_ARRAY);
		  glPopAttrib();
		}
	  }

     } break;
  }
}

Q_EXPORT_PLUGIN(SampleMeshDecoratePlugin)
